/**
* Practica Tema 5: DAYTIME UDP
*
* Redondo Calleja, Daniel
* Jimenez Prieto, Roberto
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netdb.h>      // Para getservbyname()
#include <sys/socket.h> // Para sockets
#include <netinet/in.h> // Para estructuras de sockets
#include <errno.h>

#define MAX_BUFFER 256

// Función para obtener la hora actual utilizando el comando "date"
void get_date(char *buffer, size_t longitud) {
    FILE *file;
    system("date > /tmp/date.txt");     // Ejecuta date y guarda la salida en un temporal date.txt
    file = fopen("/tmp/date.txt", "r");   // Verificamos que se ha creado date.txt
    if (file == NULL || fgets(buffer, longitud, file) == NULL) {
        perror("Error al obtener la fecha con system() o fopen()");
        exit(EXIT_FAILURE);
    }
    fclose(file);
}


int main(int argc, char *argv[]) {
    int sockfd;
    unsigned short port;
    struct sockaddr_in server_addr, client_addr;
    socklen_t client_len = sizeof(client_addr);
    char buffer[MAX_BUFFER], date_buffer[MAX_BUFFER];
    char hostname[MAX_BUFFER];

    // Obtenemos el puerto por argumento
    if (argc == 3 && strcmp(argv[1], "-p") == 0) {
        port = atoi(argv[2]);
        if (port <= 0) {
            fprintf(stderr, "Número de puerto inválido: %d\n", port);
            return EXIT_FAILURE;
        }
    // O lo obtenemos por getservbyname() dando el servicio "daytime"
    } else if (argc == 1) {
        struct servent *service = getservbyname("daytime", "udp");
        if (service == NULL) {
            fprintf(stderr, "Error al obtener el puerto para el servicio DAYTIME\n");
            return EXIT_FAILURE;
        }
        port = htons(service->s_port);
    // Si no lo hemos obtenido, notificamos el error
    } else {
        fprintf(stderr, "Error de Parametros.\nUso: %s [-p puerto]\n", argv[0]);
        return EXIT_FAILURE;
    }

    // Creamos el socket UDP
    if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        perror("Error al crear el socket");
        return EXIT_FAILURE;
    }

    // Configuramos la dirección del servidor
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = INADDR_ANY;
    server_addr.sin_port = htons(port);

    // Vinculamos el socket al puerto
    if (bind(sockfd, (struct sockaddr *)&server_addr, sizeof(server_addr)) < 0) {
        perror("Error en bind()");
        close(sockfd);
        return EXIT_FAILURE;
    }

    // Imprimimos un mensaje para verificar que se ha creado correctamente
    printf("Servidor UDP Daytime en el puerto %d\n", port);

    // Obtenemos el nombre del host
    if (gethostname(hostname, sizeof(hostname)) < 0) {
        perror("Error al obtener el nombre del servidor");
        close(sockfd);
        return EXIT_FAILURE;
    }

    while (1) {
        // Esperaramos los datagramas de los clientes
        int received_bytes = recvfrom(sockfd, buffer, MAX_BUFFER - 1, 0, (struct sockaddr *)&client_addr, &client_len);
        if (received_bytes < 0) {
            perror("Error al recibir datagrama");
            continue;
        }

        // Obtenenemos la fecha y hora actual
        get_date(date_buffer, MAX_BUFFER);

        // Prepararamos la respuesta en formato "nombre: fecha"
        snprintf(buffer, MAX_BUFFER, "%s: %s", hostname, date_buffer);

        // Enviamos la respuesta al cliente
        if (sendto(sockfd, buffer, strlen(buffer), 0, (struct sockaddr *)&client_addr, client_len) < 0) {
            perror("Error al enviar la respuesta");
        }
    }

    // Cerramos el socket y finalizamos la ejecucion
    close(sockfd);
    return 0;
}
